@@ -13,7 +13,7 @@
 package Net::STOMP::Client::IO;
 use strict;
 use warnings;
-our $VERSION = sprintf("%d.%02d", q$Revision: 1.16 $ =~ /(\d+)\.(\d+)/);
+our $VERSION = sprintf("%d.%02d", q$Revision: 1.19 $ =~ /(\d+)\.(\d+)/);
 
 #
 # Object Oriented definition
@@ -21,7 +21,9 @@ our $VERSION = sprintf("%d.%02d", q$Revision: 1.16 $ =~ /(\d+)\.(\d+)/);
 
 use Net::STOMP::Client::OO;
 our(@ISA) = qw(Net::STOMP::Client::OO);
-Net::STOMP::Client::OO::methods(qw(_socket _select _inbuf _outbuf _error));
+Net::STOMP::Client::OO::methods(qw(
+    _socket _select _error _incoming_buffer _outgoing_buffer _last_read _last_write
+));
 
 #
 # used modules
@@ -55,8 +57,8 @@ sub new : method {
     $select = IO::Select->new();
     $select->add($socket);
     $self->_select($select);
-    $self->_inbuf("");
-    $self->_outbuf("");
+    $self->_incoming_buffer("");
+    $self->_outgoing_buffer("");
     return($self);
 }
 
@@ -96,7 +98,7 @@ sub _buf2sock : method {
     # we cannot write anything if the socket is in error
     return() if $self->_error();
     # find out how much we should write
-    $towrite = length($self->{_outbuf});
+    $towrite = length($self->{_outgoing_buffer});
     $towrite = $option{size} if $option{size} and $option{size} < $towrite;
     return(0) unless $towrite > 0;
     # timer starts now
@@ -109,16 +111,17 @@ sub _buf2sock : method {
 	# write one chunk
 	$chunk = $towrite;
 	$chunk = MAX_CHUNK if $chunk > MAX_CHUNK;
-	$count = syswrite($self->_socket(), $self->{_outbuf}, $chunk);
+	$count = syswrite($self->_socket(), $self->{_outgoing_buffer}, $chunk);
 	unless (defined($count)) {
 	    $self->_error("cannot syswrite(): $!");
 	    return();
 	}
 	# update where we are
 	if ($count) {
+	    $self->_last_write(Time::HiRes::time());
 	    $written += $count;
 	    $towrite -= $count;
-	    substr($self->{_outbuf}, 0, $count) = "";
+	    substr($self->{_outgoing_buffer}, 0, $count) = "";
 	}
 	# stop if enough has been written
 	last if $towrite <= 0;
@@ -165,7 +168,8 @@ sub _sock2buf : method {
 	# read one chunk
 	$chunk = $option{size} ? $toread : MAX_CHUNK;
 	$chunk = MAX_CHUNK if $chunk > MAX_CHUNK;
-	$count = sysread($self->_socket(), $self->{_inbuf}, $chunk, length($self->{_inbuf}));
+	$count = sysread($self->_socket(), $self->{_incoming_buffer}, $chunk,
+			 length($self->{_incoming_buffer}));
 	unless (defined($count)) {
 	    $self->_error("cannot sysread(): $!");
 	    return();
@@ -176,6 +180,7 @@ sub _sock2buf : method {
 	    return();
 	}
 	# update where we are
+	$self->_last_read(Time::HiRes::time());
 	$read   += $count;
 	$toread -= $count if $option{size};
 	# stop if enough has been read
@@ -212,13 +217,13 @@ sub send_data : method {
     my($me, $result);
 
     $me = "Net::STOMP::Client::IO::send_data()";
-    $self->{_outbuf} .= $buffer;
+    $self->{_outgoing_buffer} .= $buffer;
     $result = $self->_buf2sock(timeout => $timeout);
     unless (defined($result)) {
 	Net::STOMP::Client::Error::report("%s: %s", $me, $self->_error());
 	return();
     }
-    if (length($self->{_outbuf})) {
+    if (length($self->{_outgoing_buffer})) {
 	Net::STOMP::Client::Error::report("%s: could not send all data!", $me);
 	return();
     }
@@ -273,6 +278,26 @@ This module provides Input/Output support for Net::STOMP::Client.
 It is used internally by Net::STOMP::Client and is not expected to be
 used elsewhere.
 
+=head1 FUNCTIONS
+
+This module provides the following functions and methods:
+
+=over
+
+=item new(SOCKET)
+
+create a new Net::STOMP::Client::IO object
+
+=item send_data(BUFFER[, TIMEOUT])
+
+send the data in the given buffer to the socket
+
+=item receive_data([TIMEOUT])
+
+receive data from the socket and put in the internal buffer
+
+=back
+
 =head1 AUTHOR
 
 Lionel Cons L<http://cern.ch/lionel.cons>
